function part_track_sift(fidx, im_dir, PartsPose_est_dir, trans_dir, ni, fidx_step)

% fidx: source frame index - tracker should start at this frame [1,2,3,...,N]
% im_dir: directory containing all the images
% PartsPose_est_dir: directory where pose detection results are stored
% 
% trans_dir: if the pose estimation is run on the cropped images then you
% should provide here the directory where crop-to-full image
% transformations are stored.
% 
% ni: total number of source images for which tracker should run
% fidx_step: do you want to run tracker for every frame? then fidx_step 
% should be 1
% -------------------------------------------------------------------------
% The important thing is to have the pose result in the following format.
% The script below expects the pose to be in the following form.
% variable name: partspose
% partspose dimensions, 
% rows: nParts [10 in our case]
% cols: 4 [scale_idx, rot_idx, x, y] only x,y are important for the tracker

addpath('../sift');

if nargin < 4
	use_crop = false;
	ni = 1;
	fidx_step = 1;
else
	use_crop = true;
  trans_files = dir([trans_dir, '/*mat']);
end

% ----- Every 10th Image -----
% fidx = 3;
% clc;
% clear all;
% im_dir
% PartsPose_est_dir
% trans_dir

%im_dir = '/BS/video/work/kitchen/4dishes-for-kt/activities/s07-d01-v02021/cam-002/cutting-bread/png_full/';

% directory to store tracking results
res_tracks_dir = [PartsPose_est_dir(1:end-10)];
res_tracks_dir = [res_tracks_dir, '/res_tracks/'];
if (~isdir(res_tracks_dir))
    mkdir(res_tracks_dir);
end

im_files = dir([im_dir, '/*.jpg']);
pose_files = dir([PartsPose_est_dir, '/*mat']);

total_parts = 10; % total number of parts in our pose model
parts_used = [2,3,4,5,6,7,8,9]; % these parts are being used for tracking

% *************************
track_len = 50; % how many frames forward and backward should be tracked

% crop size around the part detection that is being used for sift matching
crop_size_x = 80; 
crop_size_y = 80;
% *************************

fidx_original = fidx;

while (fidx < fidx_original+ni)

    if (fidx > length(im_files))
        break;
    end
    
for partpose_idx = fidx %start_fidx:length(im_files)
    
%     imgidx = (partpose_idx-1)*10 + 1 % We are computing PS-estimation-results for every 10th frame
   imgidx = (partpose_idx-1) + 1; % We are computing PS-estimation-results for every frame
    
    if (imgidx < (track_len+1))
        start_idx = 1;
    else
        start_idx = imgidx - track_len;
    end
    
    if (imgidx > length(im_files) - (track_len+1))
        end_idx = length(im_files)-1;
    else
        end_idx = imgidx + track_len;
    end
    
    
    clear('parts_data');
    ii = 1; % index for 'parts_data'
    
    % Load data for first image
    part_file_path = [PartsPose_est_dir, pose_files(partpose_idx).name];
    fprintf('\nloading ... %s\n', part_file_path);

    load(part_file_path);

    % if detections are needed to be transformed from crop to full images
    if (use_crop)
      % ------------------------------------------
      % Transform points back to the large image
      % ------------------------------------------
      trans_file_path = [trans_dir, trans_files(partpose_idx).name];
      T = load(trans_file_path);

      point = [];
      for j=1:total_parts % 10 parts

          homo_point = [partspose(j,3:4), 1]';

          % proj_point = inv(T.T4I) * homo_point;
          proj_point = inv(T.M) * homo_point;
          proj_point = proj_point/proj_point(3);

          partspose(j,3:4) = proj_point(1:2)';

      end
      % ------------------------------------------
    end
    
    
    partspose_1 = partspose; % contains -> [scale, rotation, x, y]
    
    
    % --- forward tracking ---
    for i = imgidx:end_idx
        
        % Load data for first image
        im_path = [im_dir, im_files(i).name];
        im_1 = imreadbw(im_path);
        [h1, w1, d1] = size(im_1);
        % --------------------------
        
        % Load data for second image
        im_path = [im_dir, im_files(i+1).name];
        im_2 = imreadbw(im_path);
        [h2, w2, d2] = size(im_2);
        % --------------------------
        
        borderX = 30;
        borderY = 30;
        
        
        for j=1:total_parts % 10 parts
            parts_data(ii).parts(j).partspose = partspose_1(j,:);
            %parts_data(ii).parts(j).bbox_data = im_1(partspose_1(j,4)-borderY : partspose_1(j,4)+borderY, partspose_1(j,3)-borderX : partspose_1(j,3)+borderX);
            parts_data(ii).parts(j).rect = [partspose_1(j,3)-borderX, partspose_1(j,4)-borderY, 2*borderX+1, 2*borderY+1];
            parts_data(ii).parts_used = parts_used;
            
        end
        
        
        for j=parts_used % only 'to be tracked' parts
            
            if (i == imgidx)
                res_x = parts_data(ii).parts(j).partspose(3);
                res_y = parts_data(ii).parts(j).partspose(4);
            else
                res_x = temp_pd(j,1); %parts_data(ii-1).parts(j).partspose(3);
                res_y = temp_pd(j,2); %parts_data(ii-1).parts(j).partspose(4);

                
                %  rectangle('position', rect, 'lineWidth', 2);
                if ( ~(isnan(res_x) || isnan(res_y)) )
                    parts_data(ii).parts(j).partspose(3:4) = [res_x, res_y];
                    %parts_data(ii).parts(j).bbox_data = im_1(result.y_est_2(jj)-borderY : result.y_est_2(jj)+borderY, result.x_est_2(jj)-borderX : result.x_est_2(jj)+borderX);
                    parts_data(ii).parts(j).rect = [res_x-borderX, res_y-borderY, 2*borderX+1, 2*borderY+1];
                    parts_data(ii).parts_used = parts_used;
                else
                    error('\n I thought it could not happen !!!\n');
                    % don't worry about 'ii-1' aceesing '0' location
                    % because this part of code is executed only when 'ii'
                    % is greater than '1'
                    parts_data(ii).parts(j).partspose(3:4) = parts_data(ii-1).parts(j).partspose(3:4);
                    res_x = parts_data(ii-1).parts(j).partspose(3);
                    res_y = parts_data(ii-1).parts(j).partspose(4);
                    %parts_data(ii).parts(j).bbox_data = parts_data(ii-1).parts(j).bbox_data;
                    parts_data(ii).parts(j).rect = parts_data(ii-1).parts(j).rect;
                    parts_data(ii).parts_used = parts_used;
                end
            end
            
            res_x = round(res_x);
            res_y = round(res_y);
            
            sx_min = max(res_x - crop_size_x, 1);
            sx_max = min(res_x + crop_size_x, w1);
            
            sy_min = max(res_y - crop_size_y, 1);
            sy_max = min(res_y + crop_size_y, h1);
            
            im_1_joint = im_1(sy_min:sy_max, sx_min:sx_max);
            im_2_joint = im_2(sy_min:sy_max, sx_min:sx_max);
%             [h1_j, w1_j, d1_j] = size(im_1_joint);
%             [h2_j, w2_j, d2_j] = size(im_2_joint);
            
            for repeat_idx = 1:3
                % Get SIFT features (contains, combined Image(I), and features positions [x,y])

                switch repeat_idx
                    case 1
                    features = get_sift_features(im_1_joint, im_2_joint, 0.005, 20, 4);
                    case 2
                    features = get_sift_features(im_1_joint, im_2_joint, 0.002, 20, 5);
                end
                features.x = bsxfun(@plus, features.x, [sx_min; sx_min-features.oj; NaN]);
                features.y = bsxfun(@plus, features.y, [sy_min; sy_min-features.oi; NaN]);

                % Keep only relevant matches/points/features
                [result] = keep_only_points_inside_the_bboxes(features.x, features.y, features.oi, features.oj, parts_data(ii), j);

                result.oi = features.oi;
                result.oj = features.oj;

                result.x_est_2 = round(result.x_est_2);
                result.y_est_2 = round(result.y_est_2);
            
                if ( (isnan(result.x_est_2) || isnan(result.y_est_2)) )
                    continue;
                else
                    break;
                end
            end
            if ( (isnan(result.x_est_2) || isnan(result.y_est_2)) )
                temp_pd(j,1) = parts_data(ii).parts(j).partspose(3);
                temp_pd(j,2) = parts_data(ii).parts(j).partspose(4);
            else
                temp_pd(j,1) = result.x_est_2;
                temp_pd(j,2) = result.y_est_2;
            end
            
        end
        
        
        ii = ii + 1; % index for 'parts_data'


        imgidx
        i
        i+1
    
        
    end
    
    
    
    
    
end

% ----------------------
% track_path_name = [res_tracks_dir, sprintf('track_init_imgidx%.6d.mat', imgidx-1)];
% try
%     save(track_path_name, 'parts_data');
% catch
%     fprintf('\n--- either forward loop didnot run or someother -save- related error ---\n');
% end
% ----------------------
% return;
% MAKE SURE 'ii' has the correct index value
% --- backward tracking ---
for i = imgidx:-1:start_idx+1

    % Load data for first image
    im_path = [im_dir, im_files(i).name];
    im_1 = imreadbw(im_path);
    % --------------------------
    
    % Load data for second image
    im_path = [im_dir, im_files(i-1).name];
    im_2 = imreadbw(im_path);
    % --------------------------
    
    borderX = 30;
    borderY = 30;

        
        for j=1:total_parts % 10 parts
            parts_data(ii).parts(j).partspose = partspose_1(j,:);
            %parts_data(ii).parts(j).bbox_data = im_1(partspose_1(j,4)-borderY : partspose_1(j,4)+borderY, partspose_1(j,3)-borderX : partspose_1(j,3)+borderX);
            parts_data(ii).parts(j).rect = [partspose_1(j,3)-borderX, partspose_1(j,4)-borderY, 2*borderX+1, 2*borderY+1];
            parts_data(ii).parts_used = parts_used;
            
        end
        
        
        for j=parts_used % only 'to be tracked' parts
            
            if (i == imgidx)
                res_x = parts_data(ii).parts(j).partspose(3);
                res_y = parts_data(ii).parts(j).partspose(4);
            else
                res_x = temp_pd(j,1); %parts_data(ii-1).parts(j).partspose(3);
                res_y = temp_pd(j,2); %parts_data(ii-1).parts(j).partspose(4);

                
                %  rectangle('position', rect, 'lineWidth', 2);
                if ( ~(isnan(res_x) || isnan(res_y)) )
                    parts_data(ii).parts(j).partspose(3:4) = [res_x, res_y];
                    %parts_data(ii).parts(j).bbox_data = im_1(result.y_est_2(jj)-borderY : result.y_est_2(jj)+borderY, result.x_est_2(jj)-borderX : result.x_est_2(jj)+borderX);
                    parts_data(ii).parts(j).rect = [res_x-borderX, res_y-borderY, 2*borderX+1, 2*borderY+1];
                    parts_data(ii).parts_used = parts_used;
                else
                    error('\n I thought it could not happen !!!\n');
                    % don't worry about 'ii-1' aceesing '0' location
                    % because this part of code is executed only when 'ii'
                    % is greater than '1'
                    parts_data(ii).parts(j).partspose(3:4) = parts_data(ii-1).parts(j).partspose(3:4);
                    res_x = parts_data(ii-1).parts(j).partspose(3);
                    res_y = parts_data(ii-1).parts(j).partspose(4);
                    %parts_data(ii).parts(j).bbox_data = parts_data(ii-1).parts(j).bbox_data;
                    parts_data(ii).parts(j).rect = parts_data(ii-1).parts(j).rect;
                    parts_data(ii).parts_used = parts_used;
                end
            end
            
            res_x = round(res_x);
            res_y = round(res_y);
            
            sx_min = max(res_x - crop_size_x, 1);
            sx_max = min(res_x + crop_size_x, w1);
            
            sy_min = max(res_y - crop_size_y, 1);
            sy_max = min(res_y + crop_size_y, h1);
            
            im_1_joint = im_1(sy_min:sy_max, sx_min:sx_max);
            im_2_joint = im_2(sy_min:sy_max, sx_min:sx_max);
            % [h1_j, w1_j, d1_j] = size(im_1_joint);
            % [h2_j, w2_j, d2_j] = size(im_2_joint);
            
            for repeat_idx = 1:3
                % Get SIFT features (contains, combined Image(I), and features positions [x,y])

                switch repeat_idx
                    case 1
                    features = get_sift_features(im_1_joint, im_2_joint, 0.005, 20, 4);
                    case 2
                    features = get_sift_features(im_1_joint, im_2_joint, 0.002, 20, 5);
                end
                features.x = bsxfun(@plus, features.x, [sx_min; sx_min-features.oj; NaN]);
                features.y = bsxfun(@plus, features.y, [sy_min; sy_min-features.oi; NaN]);

                % Keep only relevant matches/points/features
                [result] = keep_only_points_inside_the_bboxes(features.x, features.y, features.oi, features.oj, parts_data(ii), j);

                result.oi = features.oi;
                result.oj = features.oj;

                result.x_est_2 = round(result.x_est_2);
                result.y_est_2 = round(result.y_est_2);
            
                if ( (isnan(result.x_est_2) || isnan(result.y_est_2)) )
                    continue;
                else
                    break;
                end
            end
            if ( (isnan(result.x_est_2) || isnan(result.y_est_2)) )
                temp_pd(j,1) = parts_data(ii).parts(j).partspose(3);
                temp_pd(j,2) = parts_data(ii).parts(j).partspose(4);
            else
                temp_pd(j,1) = result.x_est_2;
                temp_pd(j,2) = result.y_est_2;
            end
            
        end
        
        
        ii = ii + 1; % index for 'parts_data'
    
    
    imgidx
    i
    i-1
    
end

% ----------------------
track_path_name = [res_tracks_dir, sprintf('track_init_imgidx%.6d.mat', imgidx-1)];
try
    save(track_path_name, 'parts_data');
catch
    fprintf('\n--- WARNING! either parts_data is EMPTY or some other -save- related error!!!\n');
end
% ----------------------

fidx = fidx + fidx_step;

end % while loop over fidx




end

